"""
    Pygments IRC formatter tests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

from io import StringIO

from pygments.lexers import PythonLexer
from pygments.formatters import IRCFormatter

tokensource = list(PythonLexer().get_tokens("lambda x: 123"))


def test_correct_output():
    hfmt = IRCFormatter()
    houtfile = StringIO()
    hfmt.format(tokensource, houtfile)

    assert '\x0302lambda\x03 x: \x0302123\x03\n' == houtfile.getvalue()



"""
    Basic CrystalLexer Test
    ~~~~~~~~~~~~~~~~~~~~~~~

    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import pytest

from pygments.token import Text, String, Number, Punctuation, Error, Whitespace
from pygments.lexers import CrystalLexer


@pytest.fixture(scope='module')
def lexer():
    yield CrystalLexer()


def test_numbers(lexer):
    for kind, testset in [
        (Number.Integer, '0  1  1_000_000  1u8  11231231231121312i64'),
        (Number.Float, '0.0  1.0_f32  1_f32  0f64  1e+4  1e111  1_234.567_890'),
        (Number.Bin, '0b1001_0110  0b0u8'),
        (Number.Oct, '0o17  0o7_i32'),
        (Number.Hex, '0xdeadBEEF'),
    ]:
        for fragment in testset.split():
            assert list(lexer.get_tokens(fragment + '\n')) == \
                [(kind, fragment), (Whitespace, '\n')]

    for fragment in '01  0b2  0x129g2  0o12358'.split():
        assert next(lexer.get_tokens(fragment + '\n'))[0] == Error


def test_symbols(lexer):
    for fragment in [':sym_bol', ':\u3042', ':question?']:
        assert list(lexer.get_tokens(fragment + '\n')) == \
            [(String.Symbol, fragment), (Whitespace, '\n')]

    fragment = ':"sym bol"\n'
    tokens = [
        (String.Symbol, ':"'),
        (String.Symbol, 'sym bol'),
        (String.Symbol, '"'),
        (Whitespace, '\n'),
    ]
    assert list(lexer.get_tokens(fragment)) == tokens


def test_chars(lexer):
    for fragment in ["'a'", "'я'", "'\\u{1234}'", "'\n'"]:
        assert list(lexer.get_tokens(fragment + '\n')) == \
            [(String.Char, fragment), (Whitespace, '\n')]
    assert next(lexer.get_tokens("'abc'"))[0] == Error


def test_string_escapes(lexer):
    for body in ['\\n', '\\a', '\\xff', '\\u1234', '\\000', '\\u{0}', '\\u{10AfF9}']:
        fragment = '"a' + body + 'z"\n'
        assert list(lexer.get_tokens(fragment)) == [
            (String.Double, '"'),
            (String.Double, 'a'),
            (String.Escape, body),
            (String.Double, 'z'),
            (String.Double, '"'),
            (Whitespace, '\n'),
        ]


def test_empty_percent_strings(lexer):
    for body in ['%()', '%[]', '%{}', '%<>', '%||']:
        fragment = '(' + body + ')\n'
        assert list(lexer.get_tokens(fragment)) == [
            (Punctuation, '('),
            (String.Other, body[:-1]),
            (String.Other, body[-1]),
            (Punctuation, ')'),
            (Whitespace, '\n'),
        ]



"""
    Javascript tests
    ~~~~~~~~~~~~~~~~

    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import pytest

from pygments.lexers.javascript import JavascriptLexer
from pygments.token import Number


@pytest.fixture(scope='module')
def lexer():
    yield JavascriptLexer()


@pytest.mark.parametrize(
    'text',
    (
        '1', '1.', '.1', '1.1', '1e1', '1E1', '1e+1', '1E-1', '1.e1', '.1e1',
        '0888',  # octal prefix with non-octal numbers
    )
)
def test_float_literal_positive_matches(lexer, text):
    """Test literals that should be tokenized as float literals."""
    assert list(lexer.get_tokens(text))[0] == (Number.Float, text)


@pytest.mark.parametrize('text', ('.\u0b6a', '.', '1..', '1n', '1ee', '1e',
                                  '1e-', '1e--1', '1e++1', '1e1.0'))
def test_float_literals_negative_matches(lexer, text):
    """Test text that should **not** be tokenized as float literals."""
    assert list(lexer.get_tokens(text))[0] != (Number.Float, text)


@pytest.mark.parametrize('text', ('0n', '123n'))
def test_integer_literal_positive_matches(lexer, text):
    """Test literals that should be tokenized as integer literals."""
    assert list(lexer.get_tokens(text))[0] == (Number.Integer, text)


@pytest.mark.parametrize('text', ('1N', '1', '1.0'))
def test_integer_literals_negative_matches(lexer, text):
    """Test text that should **not** be tokenized as integer literals."""
    assert list(lexer.get_tokens(text))[0] != (Number.Integer, text)


@pytest.mark.parametrize('text', ('0b01', '0B10n'))
def test_binary_literal_positive_matches(lexer, text):
    """Test literals that should be tokenized as binary literals."""
    assert list(lexer.get_tokens(text))[0] == (Number.Bin, text)


@pytest.mark.parametrize('text', ('0b0N', '0b', '0bb', '0b2'))
def test_binary_literals_negative_matches(lexer, text):
    """Test text that should **not** be tokenized as binary literals."""
    assert list(lexer.get_tokens(text))[0] != (Number.Bin, text)


@pytest.mark.parametrize('text', ('017', '071n', '0o11', '0O77n'))
def test_octal_literal_positive_matches(lexer, text):
    """Test literals that should be tokenized as octal literals."""
    assert list(lexer.get_tokens(text))[0] == (Number.Oct, text)


@pytest.mark.parametrize('text', ('01N', '089', '098n', '0o', '0OO', '0o88', '0O88n'))
def test_octal_literals_negative_matches(lexer, text):
    """Test text that should **not** be tokenized as octal literals."""
    assert list(lexer.get_tokens(text))[0] != (Number.Oct, text)


@pytest.mark.parametrize('text', ('0x01', '0Xefn', '0x0EF'))
def test_hexadecimal_literal_positive_matches(lexer, text):
    """Test literals that should be tokenized as hexadecimal literals."""
    assert list(lexer.get_tokens(text))[0] == (Number.Hex, text)


@pytest.mark.parametrize('text', ('0x0N', '0x', '0Xx', '0xg', '0xhn'))
def test_hexadecimal_literals_negative_matches(lexer, text):
    """Test text that should **not** be tokenized as hexadecimal literals."""
    assert list(lexer.get_tokens(text))[0] != (Number.Hex, text)



#!/usr/bin/env python

"""Test that syntax highlighting for USD files works correctly."""

import textwrap
import unittest

from pygments.lexers import UsdLexer
from pygments.token import Name, String, Whitespace


class _Common(unittest.TestCase):
    """A basic class that makes it easier to write unittests."""

    def setUp(self):
        """Create a fresh USD lexer class before each test runs."""
        self.lexer = UsdLexer()

    def _get(self, code):
        """Tokenize the code into its unique parts.

        :param code: The USD source code to split up.
        :type code: str

        :returns: The tokenized pieces.
        :rtype: list[:class:`pygments.token._TokenType`]

        """
        return list(self.lexer.get_tokens(code))


class Features(_Common):
    """Test that different features of USD highlight as expected."""

    def test_asset_path(self):
        """Check that a regular file path highlights correctly."""
        for path in [
            "@./some/path/to/a/file/foo.usda@",
            "@/some/path/to/a/file/foo.usda@",
            "@some/path/to/a/file/foo.usda@",
            r"@file://SPECI__Z-_ALIZED(syntax_here)?with_arbitrary#)(%*&)\characters.tar.gz@",
        ]:
            expected = [
                (String.Interpol, path),
                (Whitespace, "\n"),
            ]

            self.assertEqual(expected, self._get(path))

    def test_target_absolute(self):
        """Check that SdfPath syntax examples work correctly."""
        for code in [
            # Absolute paths
            "</some/another_one/here>",
            "</some/path/here.property_name>",
            "</some/path/here>",
            # Relative paths
            "<../some/another_one/here>",
            "<../some/path/here.property_name>",
            "<../some/path/here>",
        ]:
            self.assertEqual(
                [(Name.Namespace, code), (Whitespace, "\n")], self._get(code),
            )



"""
    Pygments Robot Framework lexer tests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import pytest

from pygments.lexers.robotframework import RobotFrameworkLexer


@pytest.fixture(scope='module')
def lexer():
    yield RobotFrameworkLexer()


def assert_same_text(lexer, text):
    """Show that lexed text does not remove any content. """
    tokens = list(lexer.get_tokens_unprocessed(text))
    output = ''.join(t[2] for t in tokens)
    assert text == output


def test_empty_brackets_after_scalar_variable(lexer):
    assert_same_text(lexer, '*** Variables ***\n'
                            '${test}[]\n')


def test_empty_brackets_after_list_variable(lexer):
    assert_same_text(lexer, '*** Variables ***\n'
                            '@{test}[]\n')


def test_empty_brackets_after_dict_variable(lexer):
    assert_same_text(lexer, '*** Variables ***\n'
                            '&{test}[]\n')



# -*- coding: utf-8 -*-
"""
    Basic ThingsDB Test
    ~~~~~~~~~~~~~~~~~~~~

    :copyright: Copyright 2006-2019 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import unittest

from pygments.token import Number, Text, Comment
from pygments.lexers import ThingsDBLexer


class ThingsDBTest(unittest.TestCase):

    def setUp(self):
        self.lexer = ThingsDBLexer()
        self.maxDiff = None

    def testNumber(self):
        fragment = u'42'
        tokens = [
            (Number.Integer, u'42'),
             (Text.Whitespace, '\n')
        ]
        self.assertEqual(tokens, list(self.lexer.get_tokens(fragment)))

    def testThingId(self):
        fragment = u'#42'
        tokens = [
            (Comment.Preproc, u'#42'),
             (Text.Whitespace, '\n')
        ]
        self.assertEqual(tokens, list(self.lexer.get_tokens(fragment)))



"""
    Data Tests
    ~~~~~~~~~~

    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import time

import pytest

from pygments.lexers.data import JsonLexer, JsonBareObjectLexer, JsonLdLexer
from pygments.token import Token, Punctuation, Text, Number, String, Keyword, \
        Name, Whitespace


@pytest.fixture(scope='module')
def lexer_json():
    yield JsonLexer()


@pytest.fixture(scope='module')
def lexer_bare():
    yield JsonBareObjectLexer()


@pytest.fixture(scope='module')
def lexer_json_ld():
    yield JsonLdLexer()


@pytest.mark.parametrize(
    'text, expected_token_types',
    (
        # Integers
        ('0', (Number.Integer,)),
        ('-1', (Number.Integer,)),
        ('1234567890', (Number.Integer,)),
        ('-1234567890', (Number.Integer,)),

        # Floats, including scientific notation
        ('123456789.0123456789', (Number.Float,)),
        ('-123456789.0123456789', (Number.Float,)),
        ('1e10', (Number.Float,)),
        ('-1E10', (Number.Float,)),
        ('1e-10', (Number.Float,)),
        ('-1E+10', (Number.Float,)),
        ('1.0e10', (Number.Float,)),
        ('-1.0E10', (Number.Float,)),
        ('1.0e-10', (Number.Float,)),
        ('-1.0E+10', (Number.Float,)),

        # Strings (escapes are tested elsewhere)
        ('""', (String.Double,)),
        ('"abc"', (String.Double,)),
        ('"ひらがな"', (String.Double,)),
        ('"123"', (String.Double,)),
        ('"[]"', (String.Double,)),
        ('"{}"', (String.Double,)),
        ('"true"', (String.Double,)),
        ('"false"', (String.Double,)),
        ('"null"', (String.Double,)),
        ('":,"', (String.Double,)),

        # Constants
        ('true', (Keyword.Constant, )),
        ('false', (Keyword.Constant, )),
        ('null', (Keyword.Constant, )),

        # Whitespace
        ('\u0020', (Whitespace,)),  # space
        ('\u000a', (Whitespace,)),  # newline
        ('\u000d', (Whitespace,)),  # carriage return
        ('\u0009', (Whitespace,)),  # tab

        # Arrays
        ('[]', (Punctuation,)),
        ('["a", "b"]', (Punctuation, String.Double, Punctuation,
                        Whitespace, String.Double, Punctuation)),

        # Objects
        ('{}', (Punctuation,)),
        ('{"a": "b"}', (Punctuation, Name.Tag, Punctuation,
                        Whitespace, String.Double, Punctuation)),
    )
)
def test_json_literals_positive_match(lexer_json, text, expected_token_types):
    """Validate that syntactically-correct JSON literals are parsed correctly."""

    tokens = list(lexer_json.get_tokens_unprocessed(text))
    assert len(tokens) == len(expected_token_types)
    assert all(token[1] is expected_token
               for token, expected_token in zip(tokens, expected_token_types))
    assert ''.join(token[2] for token in tokens) == text


@pytest.mark.parametrize(
    'text',
    (
        '"', '\\', '/', 'b', 'f', 'n', 'r', 't',
        'u0123', 'u4567', 'u89ab', 'ucdef', 'uABCD', 'uEF01',
    )
)
def test_json_object_key_escapes_positive_match(lexer_json, text):
    """Validate that escape sequences in JSON object keys are parsed correctly."""

    tokens = list(lexer_json.get_tokens_unprocessed('{"\\%s": 1}' % text))
    assert len(tokens) == 6
    assert tokens[1][1] is Name.Tag
    assert tokens[1][2] == '"\\%s"' % text


@pytest.mark.parametrize(
    'text',
    (
        '"', '\\', '/', 'b', 'f', 'n', 'r', 't',
        'u0123', 'u4567', 'u89ab', 'ucdef', 'uABCD', 'uEF01',
    )
)
def test_json_string_escapes_positive_match(lexer_json, text):
    """Validate that escape sequences in JSON string values are parsed correctly."""

    text = '"\\%s"' % text
    tokens = list(lexer_json.get_tokens_unprocessed(text))
    assert len(tokens) == 1
    assert tokens[0][1] is String.Double
    assert tokens[0][2] == text


@pytest.mark.parametrize('text', ('+\n', '0\n', '""0\n', 'a\nb\n',))
def test_json_round_trip_errors(lexer_json, text):
    """Validate that past round-trip errors never crop up again."""

    tokens = list(lexer_json.get_tokens_unprocessed(text))
    assert ''.join(t[2] for t in tokens) == text


def test_json_escape_backtracking(lexer_json):
    """Confirm that there is no catastrophic backtracking in the lexer.

    This no longer applies because the JSON lexer doesn't use regular expressions,
    but the test is included to ensure no loss of functionality now or in the future.
    """

    fragment = r'{"\u00D0000\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\63CD'
    start_time = time.time()
    list(lexer_json.get_tokens(fragment))
    assert time.time() - start_time < 1, 'The JSON lexer may have catastrophic backtracking'


@pytest.mark.parametrize(
    'keyword',
    (
        'base',
        'container',
        'context',
        'direction',
        'graph',
        'id',
        'import',
        'included',
        'index',
        'json',
        'language',
        'list',
        'nest',
        'none',
        'prefix',
        'propagate',
        'protected',
        'reverse',
        'set',
        'type',
        'value',
        'version',
        'vocab',
    )
)
def test_json_ld_keywords_positive_match(lexer_json_ld, keyword):
    """Validate that JSON-LD keywords are parsed correctly."""

    tokens = list(lexer_json_ld.get_tokens_unprocessed('{"@%s": ""}' % keyword))
    assert len(tokens) == 6
    assert tokens[1][1] is Token.Name.Decorator
    assert tokens[1][2] == '"@%s"' % keyword


@pytest.mark.parametrize(
    'keyword',
    (
        '@bogus',  # "@" does not guarantee a keyword match
        '@bases',  # Begins with the keyword "@base"
        'container',  # Matches "container" but has no leading "@"
    )
)
def test_json_ld_keywords_negative_match(lexer_json_ld, keyword):
    """Validate that JSON-LD non-keywords are parsed correctly."""

    tokens = list(lexer_json_ld.get_tokens_unprocessed('{"%s": ""}' % keyword))
    assert len(tokens) == 6
    assert tokens[1][1] is Token.Name.Tag
    assert tokens[1][2] == '"%s"' % keyword



#!/usr/bin/env python
"""
    Checker for repeated tokens
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Helper script to find suspicious lexers which produce the same token
    repeatedly, i.e. for example:

    .. code::
   
      'd'           Text
      'a'           Text
      't'           Text
      'a'           Text
      'b'           Text
      'a'           Text
      's'           Text
      'e'           Text
   
    This script has two test modes: Check for tokens repeating more often than
    a given threshold, and exclude anything but single-character tokens.
    Repeated single-character tokens are quite problematic as they result in
    bloated output and are usually an indication that someone is missing a + or *
    in the regex. 

    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""
import argparse
import os
import sys


def unpack_file(path):
    """Unpack a file into text, token pairs."""
    from collections import namedtuple
    pair = namedtuple('TextTokenPair', ['text', 'token'])
    for line in open(path).readlines():
        line = line.strip()
        if line:
            # Line can start with ' or ", so let's check which one it is
            # and find the matching one
            quotation_start = 0
            quotation_end = line.rfind(line[0])
            text = line[quotation_start+1:quotation_end]
            token = line.split()[-1]
            text = text.replace('\\n', '\n')
            text = text.replace('\\t', '\t')
            yield pair(text, token)


def check_file(path, threshold, single_only):
    current_token = ''
    current_token_repeat_count = 1

    is_suspicious = False

    for value, token in unpack_file(path):
        if single_only and len(value) > 1:
            token = ''
            current_token_repeat_count = 1
            continue

        if token != current_token:
            current_token = token
            current_token_repeat_count = 1
        else:
            current_token_repeat_count += 1
        
        if current_token_repeat_count > threshold:
            is_suspicious = True
            break

    if is_suspicious:
        print(path)

    return not is_suspicious
        

def main(args):
    errors = 0
    for dir, _, files in os.walk(args.TEST_ROOT):
        for file in files:
            if not file.endswith('.output'):
                continue
            
            path = os.path.join(dir, file)
            if not check_file(path, args.threshold, args.single):
                errors += 1

    if errors > 0:
        return 1
    return 0


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('TEST_ROOT',
        help='Root directory containing the tests')
    parser.add_argument('-t', '--threshold', type=int, default=5,
        help='Warn if a token repeats itself more often then this number.')
    parser.add_argument('-s', '--single', action='store_true', default=False,
        help='Only look at tokens matching a single character')
    args = parser.parse_args()
    sys.exit(main(args))


# -*- coding: utf-8 -*-
"""
    Basic ProcfileLexer Test
    ~~~~~~~~~~~~~~~~~~~~

    :copyright: Copyright 2006-2020 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import pytest

from pygments.token import Name, Punctuation, Text
from pygments.lexers.procfile import ProcfileLexer


@pytest.fixture(scope='module')
def lexer():
    yield ProcfileLexer()


def test_basic_line(lexer):
    text = 'task: executable --options'

    tokens = lexer.get_tokens(text)

    for index, token in enumerate(tokens):
        if index == 0:
            assert token == (Name.Label, 'task')
        elif index == 1:
            assert token == (Punctuation, ':')
        else:
            assert token[0] in (Text, Text.Whitespace)


def test_environment_variable(lexer):
    text = '$XDG_SESSION_PATH'

    token = list(lexer.get_tokens(text))[0]

    assert token == (Name.Variable, text)



"""
    Basic JavaLexer Test
    ~~~~~~~~~~~~~~~~~~~~

    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import time

import pytest

from pygments.token import String
from pygments.lexers import JavaLexer


@pytest.fixture(scope='module')
def lexer():
    yield JavaLexer()


@pytest.mark.parametrize(
    'text',
    (
        '""', '"abc"', '"ひらがな"', '"123"',
        '"\\\\"', '"\\t"' '"\\""',
    ),
)
def test_string_literals_positive_match(lexer, text):
    """Test positive matches for string literals."""
    tokens = list(lexer.get_tokens_unprocessed(text))
    assert all([token is String for _, token, _ in tokens])
    assert ''.join([value for _, _, value in tokens]) == text


def test_string_literals_backtracking(lexer):
    """Test catastrophic backtracking for string literals."""
    start_time = time.time()
    list(lexer.get_tokens_unprocessed('"' + '\\' * 100))
    assert time.time() - start_time < 1, 'possible backtracking bug'



"""
    HTML Lexer Tests
    ~~~~~~~~~~~~~~~~

    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import time

import pytest

from pygments.lexers.html import HtmlLexer
from pygments.token import Token

MAX_HL_TIME = 10


@pytest.fixture(scope='module')
def lexer_html():
    yield HtmlLexer()


def test_happy_javascript_fragment(lexer_html):
    """valid, even long Javascript fragments should still get parsed ok"""

    fragment = "<script type=\"text/javascript\">"+"alert(\"hi\");"*2000+"</script>"
    start_time = time.time()
    tokens = list(lexer_html.get_tokens(fragment))
    assert all(x[1] != Token.Error for x in tokens)
    assert time.time() - start_time < MAX_HL_TIME, \
        'The HTML lexer might have an expensive happy-path script case'


def test_happy_css_fragment(lexer_html):
    """valid, even long CSS fragments should still get parsed ok"""

    fragment = "<style>"+".ui-helper-hidden{display:none}"*2000+"</style>"
    start_time = time.time()
    tokens = list(lexer_html.get_tokens(fragment))
    assert all(x[1] != Token.Error for x in tokens)
    assert time.time() - start_time < MAX_HL_TIME, \
        'The HTML lexer might have an expensive happy-path style case'


def test_long_unclosed_javascript_fragment(lexer_html):
    """unclosed, long Javascript fragments should parse quickly"""

    reps = 2000
    fragment = "<script type=\"text/javascript\">"+"alert(\"hi\");"*reps
    start_time = time.time()
    tokens = list(lexer_html.get_tokens(fragment))
    assert time.time() - start_time < MAX_HL_TIME, \
        'The HTML lexer might have an expensive error script case'
    tokens_intro = [
        (Token.Punctuation, '<'),
        (Token.Name.Tag, 'script'),
        (Token.Text, ' '),
        (Token.Name.Attribute, 'type'),
        (Token.Operator, '='),
        (Token.Literal.String, '"text/javascript"'),
        (Token.Punctuation, '>'),
    ]
    tokens_body = [
        (Token.Name.Other, 'alert'),
        (Token.Punctuation, '('),
        (Token.Literal.String.Double, '"hi"'),
        (Token.Punctuation, ')'),
        (Token.Punctuation, ';'),
    ]

    # make sure we get the right opening tokens
    assert tokens[:len(tokens_intro)] == tokens_intro
    # and make sure we get the right body tokens even though the script is
    # unclosed
    assert tokens[len(tokens_intro):-1] == tokens_body * reps
    # and of course, the newline we get for free from get_tokens
    assert tokens[-1] == (Token.Text, "\n")


def test_long_unclosed_css_fragment(lexer_html):
    """unclosed, long CSS fragments should parse quickly"""

    reps = 2000
    fragment = "<style>"+".ui-helper-hidden{display:none}"*reps
    start_time = time.time()
    tokens = list(lexer_html.get_tokens(fragment))
    assert time.time() - start_time < MAX_HL_TIME, \
        'The HTML lexer might have an expensive error style case'

    tokens_intro = [
        (Token.Punctuation, '<'),
        (Token.Name.Tag, 'style'),
        (Token.Punctuation, '>'),
    ]
    tokens_body = [
        (Token.Punctuation, '.'),
        (Token.Name.Class, 'ui-helper-hidden'),
        (Token.Punctuation, '{'),
        (Token.Keyword, 'display'),
        (Token.Punctuation, ':'),
        (Token.Keyword.Constant, 'none'),
        (Token.Punctuation, '}'),
    ]

    # make sure we get the right opening tokens
    assert tokens[:len(tokens_intro)] == tokens_intro
    # and make sure we get the right body tokens even though the style block is
    # unclosed
    assert tokens[len(tokens_intro):-1] == tokens_body * reps
    # and of course, the newline we get for free from get_tokens
    assert tokens[-1] == (Token.Text.Whitespace, "\n")


def test_unclosed_fragment_with_newline_recovery(lexer_html):
    """unclosed Javascript fragments should recover on the next line"""

    fragment = "<script type=\"text/javascript\">"+"alert(\"hi\");"*20+"\n<div>hi</div>"
    tokens = list(lexer_html.get_tokens(fragment))
    recovery_tokens = [
        (Token.Punctuation, '<'),
        (Token.Name.Tag, 'div'),
        (Token.Punctuation, '>'),
        (Token.Text, 'hi'),
        (Token.Punctuation, '<'),
        (Token.Punctuation, '/'),
        (Token.Name.Tag, 'div'),
        (Token.Punctuation, '>'),
        (Token.Text, '\n'),
    ]
    assert tokens[-1*len(recovery_tokens):] == recovery_tokens



"""
    Pygments RTF formatter tests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

from io import StringIO

from pygments.formatters import RtfFormatter
from pygments.lexers.special import TextLexer


foot = (r'\par' '\n' r'}')


def _escape(string):
    return string.replace("\n", r"\n")


def _build_message(*args, **kwargs):
    string = kwargs.get('string', None)
    t = _escape(kwargs.get('t', ''))
    expected = _escape(kwargs.get('expected', ''))
    result = _escape(kwargs.get('result', ''))

    if string is None:
        string = ("The expected output of '{t}'\n"
                  "\t\tShould be '{expected}'\n"
                  "\t\tActually outputs '{result}'\n"
                  "\t(WARNING: Partial Output of Result!)")

    end = -len(_escape(foot))
    start = end - len(expected)

    return string.format(t=t,
                         result = result[start:end],
                         expected = expected)


def format_rtf(t):
    tokensource = list(TextLexer().get_tokens(t))
    fmt = RtfFormatter()
    buf = StringIO()
    fmt.format(tokensource, buf)
    result = buf.getvalue()
    buf.close()
    return result


def test_rtf_header():
    t = ''
    result = format_rtf(t)
    expected = r'{\rtf1\ansi\uc0'
    msg = ("RTF documents are expected to start with '{expected}'\n"
           "\t\tStarts intead with '{result}'\n"
           "\t(WARNING: Partial Output of Result!)".format(
               expected=expected,
               result=result[:len(expected)]))
    assert result.startswith(expected), msg


def test_rtf_footer():
    t = ''
    result = format_rtf(t)
    expected = ''
    msg = ("RTF documents are expected to end with '{expected}'\n"
           "\t\tEnds intead with '{result}'\n"
           "\t(WARNING: Partial Output of Result!)".format(
               expected=_escape(expected),
               result=_escape(result[-len(expected):])))
    assert result.endswith(expected+foot), msg


def test_ascii_characters():
    t = 'a b c d ~'
    result = format_rtf(t)
    expected = (r'a b c d ~')
    msg = _build_message(t=t, result=result, expected=expected)
    assert result.endswith(expected+foot), msg


def test_escape_characters():
    t = '\\ {{'
    result = format_rtf(t)
    expected = r'\\ \{\{'
    msg = _build_message(t=t, result=result, expected=expected)
    assert result.endswith(expected+foot), msg


def test_single_characters():
    t = 'â € ¤ каждой'
    result = format_rtf(t)
    expected = (r'{\u226} {\u8364} {\u164} '
                r'{\u1082}{\u1072}{\u1078}{\u1076}{\u1086}{\u1081}')
    msg = _build_message(t=t, result=result, expected=expected)
    assert result.endswith(expected+foot), msg


def test_double_characters():
    t = 'က 힣 ↕ ↕︎ 鼖'
    result = format_rtf(t)
    expected = (r'{\u4096} {\u55203} {\u8597} '
                r'{\u8597}{\u65038} {\u55422}{\u56859}')
    msg = _build_message(t=t, result=result, expected=expected)
    assert result.endswith(expected+foot), msg



import lxml.html
import lxml.etree
import pytest

def _serialize(t):
    for a, e in lxml.etree.iterwalk(t, events=("start", "end"),):
        text = e.text.strip() if e.text else ""
        yield (a, e.tag, repr(text), ', '.join([k[0]+':'+k[1] for k in sorted(e.attrib.items(), key = lambda x: x[0])]))

def structural_diff(a, b):
    """Check if there is a structural difference between two HTML files."""
    a_s = _serialize(lxml.html.fromstring(a))
    b_s = _serialize(lxml.html.fromstring(b))
    
    for e, f in zip(a_s, b_s):
        print(e, f)
        assert e == f


"""
    Pygments Markdown lexer tests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import pytest
from pygments.token import Generic, Token, String

from pygments.lexers.markup import MarkdownLexer


@pytest.fixture(scope='module')
def lexer():
    yield MarkdownLexer()


def assert_same_text(lexer, text):
    """Show that lexed markdown does not remove any content. """
    tokens = list(lexer.get_tokens_unprocessed(text))
    output = ''.join(t[2] for t in tokens)
    assert text == output


def test_code_fence(lexer):
    assert_same_text(lexer, r'```\nfoo\n```\n')


def test_code_fence_gsm(lexer):
    assert_same_text(lexer, r'```markdown\nfoo\n```\n')


def test_code_fence_gsm_with_no_lexer(lexer):
    assert_same_text(lexer, r'```invalid-lexer\nfoo\n```\n')


def test_invalid_atx_heading(lexer):
    fragments = (
        '#',
        'a #',
        '*#',
    )

    for fragment in fragments:
        for token, _ in lexer.get_tokens(fragment):
            assert token != Generic.Heading


def test_atx_heading(lexer):
    fragments = (
        '#Heading',
        '# Heading',
        '# Another heading',
        '# Another # heading',
        '# Heading #',
    )

    for fragment in fragments:
        tokens = [
            (Generic.Heading, fragment),
            (Token.Text, '\n'),
        ]
        assert list(lexer.get_tokens(fragment)) == tokens


def test_invalid_atx_subheading(lexer):
    fragments = (
        '##',
        'a ##',
        '*##',
        '####### too many hashes'
    )

    for fragment in fragments:
        for token, _ in lexer.get_tokens(fragment):
            assert token != Generic.Subheading


def test_atx_subheading(lexer):
    fragments = (
        '##Subheading',
        '## Subheading',
        '### Subheading',
        '#### Subheading',
        '##### Subheading',
        '###### Subheading',
        '## Another subheading',
        '## Another ## subheading',
        '###### Subheading #',
        '###### Subheading ######',
    )

    for fragment in fragments:
        tokens = [
            (Generic.Subheading, fragment),
            (Token.Text, '\n'),
        ]
        assert list(lexer.get_tokens(fragment)) == tokens


def test_invalid_setext_heading(lexer):
    fragments = (
        'Heading\n',
        'Heading\n_',
        'Heading\n =====',
        'Heading\na=====',
        '=====',
        '\n=\n',
        'Heading\n=====Text'
    )

    for fragment in fragments:
        for token, _ in lexer.get_tokens(fragment):
            assert token != Generic.Heading


def test_setext_heading(lexer):
    fragments = (
        'Heading\n=',
        'Heading\n=======',
        'Heading\n==========',
    )

    for fragment in fragments:
        tokens = [
            (Generic.Heading, fragment.split('\n')[0]),
            (Token.Text, '\n'),
            (Generic.Heading, fragment.split('\n')[1]),
            (Token.Text, '\n'),
        ]
        assert list(lexer.get_tokens(fragment)) == tokens


def test_invalid_setext_subheading(lexer):
    fragments = (
        'Subheading\n',
        'Subheading\n_',
        'Subheading\n -----',
        'Subheading\na-----',
        '-----',
        '\n-\n',
        'Subheading\n-----Text'
    )

    for fragment in fragments:
        for token, _ in lexer.get_tokens(fragment):
            assert token != Generic.Subheading


def test_setext_subheading(lexer):
    fragments = (
        'Subheading\n-',
        'Subheading\n----------',
        'Subheading\n-----------',
    )

    for fragment in fragments:
        tokens = [
            (Generic.Subheading, fragment.split('\n')[0]),
            (Token.Text, '\n'),
            (Generic.Subheading, fragment.split('\n')[1]),
            (Token.Text, '\n'),
        ]
        assert list(lexer.get_tokens(fragment)) == tokens


def test_invalid_code_block(lexer):
    fragments = (
        '```code```',
        'prefix not allowed before ```\ncode block\n```'
        '   code',
    )

    for fragment in fragments:
        for token, _ in lexer.get_tokens(fragment):
            assert token != String.Backtick



"""
    Devicetree Lexer Tests
    ~~~~~~~~~~~~~~~~~~~~~~

    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import pytest

from pygments.lexers.devicetree import DevicetreeLexer
from pygments.token import Token


@pytest.fixture(scope="module")
def lexer():
    yield DevicetreeLexer()


@pytest.mark.parametrize(
    "fragment",
    (
        'nodelabel: node@0 { foo = "bar"; };',
        'nodelabel: node { foo = "bar"; };',
        'nodelabel0: nodelabel1: node@0 { foo = "bar"; };',
    ),
)
def test_fragment_out_of_root_node(lexer, fragment):
    """Validate that a devicetree fragment out of a root node is parsed correctly."""

    tokens = list(lexer.get_tokens(fragment))
    assert all(x[0] != Token.Error for x in tokens)
